# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""

import numpy as np
import logging

from pyspark.sql import SparkSession
import time
from pyspark.sql.types import *
from pyspark.sql.types import StringType, IntegerType
from pyspark.sql import functions as func


def execute(max_path, project_name, cpa_gyc, test_out_path):
    spark = SparkSession.builder \
        .master("yarn") \
        .appName("sparkOutlier") \
        .config("spark.driver.memory", "1g") \
        .config("spark.executor.cores", "1") \
        .config("spark.executor.instance", "2") \
        .config("spark.executor.memory", "2g") \
        .getOrCreate()

    # logging配置
    logger = logging.getLogger("log")
    logger.setLevel(level=logging.INFO)
    file_handler = logging.FileHandler('job1_hospital_mapping.log','w')
    file_handler.setLevel(level=logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - [line:%(lineno)d] - %(levelname)s - %(message)s')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)

    logger.info('job1_hospital_mapping')

    # 输入输出
    universe_path = max_path + "/" + project_name + "/universe_base"
    cpa_pha_mapping_path = max_path + "/" + project_name + "/cpa_pha_mapping"
    raw_data_path = max_path + "/" + project_name + "/raw_data"
    hospital_mapping_out_path = test_out_path + "/" + project_name + "/hospital_mapping_out"

    # =========== 数据检查 =============
    logger.info('数据检查-start')

    # 存储文件的缺失列
    misscols_dict = {}

    # universe file
    # universe_path = "/common/projects/max/Sankyo/universe_base"
    universe = spark.read.parquet(universe_path)
    colnames_universe = universe.columns
    misscols_dict.setdefault("universe", [])
    if ("City_Tier" not in colnames_universe) and ("CITYGROUP" not in colnames_universe) and ("City_Tier_2010" not in colnames_universe):
        misscols_dict["universe"].append("City_Tier/CITYGROUP")
    if ("Panel_ID" not in colnames_universe) and ("PHA" not in colnames_universe):
        misscols_dict["universe"].append("Panel_ID/PHA")
    if ("Hosp_name" not in colnames_universe) and ("HOSP_NAME" not in colnames_universe):
        misscols_dict["universe"].append("Hosp_name/HOSP_NAME")
    if "City" not in colnames_universe:
        misscols_dict["universe"].append("City")
    if "Province" not in colnames_universe:
        misscols_dict["universe"].append("Province")

    # cpa_pha_mapping file
    # cpa_pha_mapping_path = "/common/projects/max/Sankyo/cpa_pha_mapping"
    cpa_pha_mapping = spark.read.parquet(cpa_pha_mapping_path)
    colnames_map = cpa_pha_mapping.columns
    misscols_dict.setdefault("cpa_pha_mapping", [])
    if "推荐版本" not in colnames_map:
        misscols_dict["cpa_pha_mapping"].append("推荐版本")
    if ("ID" not in colnames_map) and ("BI_hospital_code" not in colnames_map):
        misscols_dict["cpa_pha_mapping"].append("ID/BI_hospital_code")
    if ("PHA" not in colnames_map) and ("PHA_ID_x" not in colnames_map):
        misscols_dict["cpa_pha_mapping"].append("PHA")

    # raw_data file
    # raw_data_path = "/common/projects/max/Sankyo/raw_data"
    raw_data = spark.read.parquet(raw_data_path)
    colnames_raw_data = raw_data.columns
    misscols_dict.setdefault("raw_data", [])
    if ("Units" not in colnames_raw_data) and ("数量（支/片）" not in colnames_raw_data) and ("最小制剂单位数量" not in colnames_raw_data) and ("total_units" not in colnames_raw_data) and ("SALES_QTY" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Units")
    if ("Sales" not in colnames_raw_data) and ("金额（元）" not in colnames_raw_data) and ("金额" not in colnames_raw_data) and ("sales_value__rmb_" not in colnames_raw_data) and ("SALES_VALUE" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Sales")
    if ("year_month" not in colnames_raw_data) and ("Yearmonth" not in colnames_raw_data) and ("YM" not in colnames_raw_data) and ("Date" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about year_month")
    if "ID" not in colnames_raw_data:
        misscols_dict["raw_data"].append("ID")
    if ("Molecule" not in colnames_raw_data) and ("通用名" not in colnames_raw_data) and ("药品名称" not in colnames_raw_data) and ("molecule_name" not in colnames_raw_data) and ("MOLE_NAME" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Molecule")
    if ("Brand" not in colnames_raw_data) and ("商品名" not in colnames_raw_data) and ("药品商品名" not in colnames_raw_data) and ("product_name" not in colnames_raw_data) and ("PRODUCT_NAME" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Brand")
    if ("Specifications" not in colnames_raw_data) and ("规格" not in colnames_raw_data) and ("pack_description" not in colnames_raw_data) and ("SPEC" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Specifications")
    if ("Form" not in colnames_raw_data) and ("剂型" not in colnames_raw_data) and ("formulation_name" not in colnames_raw_data) and ("DOSAGE" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Form")
    if ("Manufacturer" not in colnames_raw_data) and ("生产企业" not in colnames_raw_data) and ("company_name" not in colnames_raw_data) and ("MANUFACTURER_NAME" not in colnames_raw_data):
        misscols_dict["raw_data"].append("about Manufacturer")

    # 判断输入文件是否有缺失列
    for eachfile in misscols_dict.keys():
        if len(misscols_dict[eachfile]) == 0:
            del misscols_dict[eachfile]
    # 如果有缺失列，则报错，停止运行
    if misscols_dict:
        logger.error('miss columns: %s' % (misscols_dict))
        raise ValueError('miss columns: %s' % (misscols_dict))

    logger.info('数据检查-Pass')

    # =========== 数据执行 =============
    logger.info('数据执行-start')

    # 1. 首次补数

    # read_universe
    for col in universe.columns:
        if col in ["City_Tier", "CITYGROUP"]:
            universe = universe.withColumnRenamed(col, "City_Tier_2010")
    universe = universe.withColumnRenamed("Panel_ID", "PHA") \
        .withColumnRenamed("Hosp_name", "HOSP_NAME")

    universe = universe.withColumn("City_Tier_2010", universe["City_Tier_2010"].cast(StringType()))

    PHA_city_in_universe = universe.select("PHA", "City", "City_Tier_2010").distinct()

    # 1.2 读取CPA与PHA的匹配关系:
    # map_cpa_pha
    cpa_pha_mapping = cpa_pha_mapping.filter(cpa_pha_mapping["推荐版本"] == 1) \
        .withColumnRenamed("ID", "BI_hospital_code") \
        .withColumnRenamed("PHA", "PHA_ID_x") \
        .select("PHA_ID_x", "BI_hospital_code").distinct()
    cpa_pha_mapping = cpa_pha_mapping.join(universe.select("PHA", "Province", "City"),
                                           cpa_pha_mapping.PHA_ID_x == universe.PHA,
                                           how="left") \
        .drop("PHA")

    # 1.3 读取原始样本数据:
    # read_raw_data
    raw_data = raw_data.withColumnRenamed("ID", "BI_Code") \
        .drop("Province", "City")
    raw_data = raw_data.join(cpa_pha_mapping.select("PHA_ID_x", "BI_hospital_code", 'Province', 'City'),
                             raw_data.BI_Code == cpa_pha_mapping.BI_hospital_code,
                             how="left")

    # format_raw_data
    # cpa_gyc = True
    for col in raw_data.columns:
        if col in ["数量（支/片）", "最小制剂单位数量", "total_units", "SALES_QTY"]:
            raw_data = raw_data.withColumnRenamed(col, "Units")
        if col in ["金额（元）", "金额", "sales_value__rmb_", "SALES_VALUE"]:
            raw_data = raw_data.withColumnRenamed(col, "Sales")
        if col in ["Yearmonth", "YM", "Date"]:
            raw_data = raw_data.withColumnRenamed(col, "year_month")
        if col in ["年", "年份", "YEAR"]:
            raw_data = raw_data.withColumnRenamed(col, "Year")
        if col in ["月", "月份", "MONTH"]:
            raw_data = raw_data.withColumnRenamed(col, "Month")
        if col in ["医院编码", "BI_Code", "HOSP_CODE"]:
            raw_data = raw_data.withColumnRenamed(col, "ID")
        if col in ["通用名", "药品名称", "molecule_name", "MOLE_NAME"]:
            raw_data = raw_data.withColumnRenamed(col, "Molecule")
        if col in ["商品名", "药品商品名", "product_name", "PRODUCT_NAME"]:
            raw_data = raw_data.withColumnRenamed(col, "Brand")
        if col in ["规格", "pack_description", "SPEC"]:
            raw_data = raw_data.withColumnRenamed(col, "Specifications")
        if col in ["剂型", "formulation_name", "DOSAGE"]:
            raw_data = raw_data.withColumnRenamed(col, "Form")
        if col in ["包装数量", "包装规格", "PACK_QTY"]:
            raw_data = raw_data.withColumnRenamed(col, "Pack_Number")
        if col in ["生产企业", "company_name", "MANUFACTURER_NAME"]:
            raw_data = raw_data.withColumnRenamed(col, "Manufacturer")
        if col in ["省份", "省", "省/自治区/直辖市", "province_name", "PROVINCE_NAME"]:
            raw_data = raw_data.withColumnRenamed(col, "Province")
        if col in ["城市", "city_name", "CITY_NAME"]:
            raw_data = raw_data.withColumnRenamed(col, "City")
        if col in ["PHA_ID_x", "PHA_ID"]:
            raw_data = raw_data.withColumnRenamed(col, "PHA")

    if (cpa_gyc == "True"):
        def distinguish_cpa_gyc(col, gyc_hospital_id_length):
            # gyc_hospital_id_length是国药诚信医院编码长度，一般是7位数字，cpa医院编码一般是6位数字。医院编码长度可以用来区分cpa和gyc
            return (func.length(col) < gyc_hospital_id_length)

        raw_data = raw_data.withColumn("ID", raw_data["ID"].cast(StringType()))
        raw_data = raw_data.withColumn("ID", func.when(distinguish_cpa_gyc(raw_data.ID, 7), func.lpad(raw_data.ID, 6, "0")).
                                       otherwise(func.lpad(raw_data.ID, 7, "0")))
    if "year_month" in raw_data.columns:
        raw_data = raw_data.withColumn("year_month", raw_data["year_month"].cast(IntegerType()))
    if "Month" not in raw_data.columns:
        raw_data = raw_data.withColumn("Month", raw_data.year_month % 100)
    if "Pack_Number" not in raw_data.columns:
        raw_data = raw_data.withColumn("Pack_Number", func.lit(0))
    if "Year" not in raw_data.columns:
        raw_data = raw_data.withColumn("Year", (raw_data.year_month - raw_data.Month) / 100)

    raw_data = raw_data.withColumn("Year", raw_data["Year"].cast(IntegerType())) \
        .withColumn("Month", raw_data["Month"].cast(IntegerType()))

    raw_data.persist()

    raw_data = raw_data.join(PHA_city_in_universe, on=["PHA", "City"], how="left")

    # 与原R流程运行的结果比较正确性
    hospital_mapping_out = raw_data.repartition(2)
    hospital_mapping_out.write.format("parquet") \
        .mode("overwrite").save(hospital_mapping_out_path)

    raw_data.show(2)

    logger.info('数据执行-Finish')

    # =========== 数据验证 =============

    if True:
        logger.info('数据验证-start')
        
        my_out = raw_data
        
        R_out_path = "/user/ywyuan/max/Sankyo/Rout/hospital_mapping_out"
        R_out = spark.read.parquet(R_out_path)
                    
        # 检查内容：列缺失，列的类型，列的值
        for colname, coltype in R_out.dtypes:
            # 列是否缺失
            if colname not in my_out.columns:
                print ("miss columns:", colname)
            else:
                # 数据类型检查
                if my_out.select(colname).dtypes[0][1] != coltype:
                    logger.warning("different type columns: " + colname + ", " + my_out.select(colname).dtypes[0][1] + ", " + "right type: " + coltype)
            
                # 数值列的值检查
                if coltype == "double" or coltype == "int":
                    # year_month, Pack_Number, Sales, Units, Units_Box, BI_hospital_code, Month, Year
                    sum_my_out = my_out.groupBy().sum(colname).toPandas().iloc[0, 0]
                    sum_R = R_out.groupBy().sum(colname).toPandas().iloc[0, 0]
                    # print (colname, sum_raw_data, sum_R)
                    if (sum_my_out - sum_R) != 0:
                        logger.warning("different value(sum) columns: " + colname + ", " + str(sum_my_out) + ", " + "right value: " + str(sum_R))

        logger.info('数据验证-Finish')
        
    # =========== return =============     
    return raw_data







from phjob import execute
import click


@click.command()
@click.option('--max_path')
@click.option('--project_name')
@click.option('--cpa_gyc')
@click.option('--test_out_path')
def debug_execute(max_path, project_name, cpa_gyc, test_out_path):
    execute(max_path, project_name, cpa_gyc, test_out_path)


if __name__ == '__main__':
    debug_execute()



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""



def execute(a, b):
	"""
		please input your code below
	"""
	print("this is job one, alfred yang")



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""
from phjob import execute
import click


@click.command()
@click.option('--a')
@click.option('--b')
def debug_execute(a, b):
	execute(a, b)


if __name__ == '__main__':
    debug_execute()




# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""
import click
import numpy as np
import logging

from pyspark.sql import SparkSession
import time
from pyspark.sql.types import *
from pyspark.sql.types import StringType, IntegerType
from pyspark.sql import functions as func

def execute(max_path, max_path_local, project_name, minimum_product_columns, minimum_product_sep,minimum_product_newname, need_cleaning_cols, test_out_path):
    spark = SparkSession.builder \
        .master("yarn") \
        .appName("sparkOutlier") \
        .config("spark.driver.memory", "1g") \
        .config("spark.executor.cores", "1") \
        .config("spark.executor.instance", "2") \
        .config("spark.executor.memory", "2g") \
        .getOrCreate()
        
    # logging配置
    logger = logging.getLogger("log")
    logger.setLevel(level=logging.INFO)
    file_handler = logging.FileHandler('job2_product_mapping.log','w')
    file_handler.setLevel(level=logging.INFO)
    formatter = logging.Formatter('%(asctime)s - %(name)s - [line:%(lineno)d] - %(levelname)s - %(message)s')
    file_handler.setFormatter(formatter)
    logger.addHandler(file_handler)
    
    logger.info('job2_product_mapping')
        
    # 输入输出
    product_map_path = max_path + "/" + project_name + "/prod_mapping"
    hospital_mapping_out_path = test_out_path + "/" + project_name + "/hospital_mapping_out"
    product_mapping_out_path = test_out_path + "/" + project_name + "/product_mapping_out"
    need_cleaning_path = max_path_local + "/" + project_name + "/need_cleaning.xlsx"
    
    # =========== 数据检查 =============
    logger.info('数据检查-start')
    
    # 存储文件的缺失列
    misscols_dict = {}

    # product_map file
    # product_map_path = "/common/projects/max/Sankyo/prod_mapping"
    product_map = spark.read.parquet(product_map_path)
    colnames_product_map = product_map.columns
    misscols_dict.setdefault("universe", [])
    if ("标准通用名" not in colnames_product_map) and ("通用名"  not in colnames_product_map):
        misscols_dict["product_map"].append("标准通用名")
    if "min1" not in colnames_product_map:
        misscols_dict["product_map"].append("min1")
    if "min2" not in colnames_product_map:
        misscols_dict["product_map"].append("min2")
    if "标准商品名" not in colnames_product_map:
        misscols_dict["product_map"].append("标准商品名") 
    
    # 判断输入文件是否有缺失列
    for eachfile in misscols_dict.keys():
        if len(misscols_dict[eachfile]) == 0:
            del misscols_dict[eachfile]
    # 如果有缺失列，则报错，停止运行
    if misscols_dict:
        logger.error('miss columns: %s' % (misscols_dict))
        raise ValueError('miss columns: %s' % (misscols_dict))
        
    logger.info('数据检查-Pass')
    
    # =========== 数据执行 =============
    logger.info('数据执行-start')
    
    # raw_data_job1_out_path = "/user/ywyuan/max/Sankyo/raw_data_job1_out"
    raw_data = spark.read.parquet(hospital_mapping_out_path)
    raw_data = raw_data.withColumn("Brand", func.when(func.isnull(raw_data.Brand), raw_data.Molecule).
                                   otherwise(raw_data.Brand))

    # concat_multi_cols
    # minimum_product_columns = ["Brand", "Form", "Specifications", "Pack_Number", "Manufacturer"]
    # minimum_product_sep = ""
    # minimum_product_newname = "min1"
    minimum_product_columns = minimum_product_columns.split(", ")
    for colname, coltype in raw_data.dtypes:
        if coltype == "logical":
            raw_data = raw_data.withColumn(colname, raw_data[colname].cast(StringType()))

    raw_data = raw_data.withColumn("tmp", func.when(func.isnull(raw_data[minimum_product_columns[0]]), func.lit("NA")).
                                   otherwise(raw_data[minimum_product_columns[0]]))

    for col in minimum_product_columns[1:]:
        raw_data = raw_data.withColumn("tmp", func.concat(
            raw_data["tmp"],
            func.lit(minimum_product_sep),
            func.when(func.isnull(raw_data[col]), func.lit("NA")).otherwise(raw_data[col])))

    raw_data = raw_data.withColumnRenamed("tmp", minimum_product_newname)

    # product_map
    product_map = product_map.withColumnRenamed("标准通用名", "通用名") \
        .withColumnRenamed("标准途径", "std_route")
    if "std_route" not in product_map.columns:
        product_map = product_map.withColumn("std_route", func.lit(''))

    product_map_for_needclean = product_map.select("min1").distinct()
    product_map_for_rawdata = product_map.select("min1", "min2", "通用名", "std_route", "标准商品名").distinct()
    
    # 输出待清洗
    # need_cleaning_cols = ["Molecule", "min1", "Route", "Corp"]
    need_cleaning_cols = need_cleaning_cols.split(", ")
    need_cleaning_cols[1:1] = minimum_product_columns
    need_cleaning = raw_data.join(product_map_for_needclean, on="min1", how="left_anti") \
        .select(need_cleaning_cols) \
        .distinct()
    logger.info('待清洗行数: ' + str(need_cleaning.count()))

    # need_cleaning_path = "/user/ywyuan/max/Sankyo/need_cleaning.xlsx"
    if need_cleaning.count() > 0:
        need_cleaning = need_cleaning.toPandas()
        need_cleaning.to_excel(need_cleaning_path)
        #print("已输出待清洗文件至:", need_cleaning_path)
        logger.info('已输出待清洗文件至:  ' + need_cleaning_path)

    raw_data = raw_data.join(product_map_for_rawdata, on="min1", how="left") \
        .drop("S_Molecule") \
        .withColumnRenamed("通用名", "S_Molecule")

    product_mapping_out = raw_data.repartition(2)
    product_mapping_out.write.format("parquet") \
        .mode("overwrite").save(product_mapping_out_path)

    raw_data.show(2)
    
    logger.info('数据执行-Finish')
    
    # =========== 数据验证 =============
    # 与原R流程运行的结果比较正确性
    if True:
        logger.info('数据验证-start')
        
        my_out = raw_data
        
        R_out_path = "/user/ywyuan/max/Sankyo/Rout/product_mapping_out"
        R_out = spark.read.parquet(R_out_path)
                    
        # 检查内容：列缺失，列的类型，列的值
        for colname, coltype in R_out.dtypes:
            # 列是否缺失
            if colname not in my_out.columns:
                print ("miss columns:", colname)
            else:
                # 数据类型检查
                if my_out.select(colname).dtypes[0][1] != coltype:
                    logger.warning("different type columns: " + colname + ", " + my_out.select(colname).dtypes[0][1] + ", " + "right type: " + coltype)
            
                # 数值列的值检查
                if coltype == "double" or coltype == "int":
                    # year_month, Pack_Number, Sales, Units, Units_Box, BI_hospital_code, Month, Year
                    sum_my_out = my_out.groupBy().sum(colname).toPandas().iloc[0, 0]
                    sum_R = R_out.groupBy().sum(colname).toPandas().iloc[0, 0]
                    # print (colname, sum_raw_data, sum_R)
                    if (sum_my_out - sum_R) != 0:
                        logger.warning("different value(sum) columns: " + colname + ", " + str(sum_my_out) + ", " + "right value: " + str(sum_R))
                    
        logger.info('数据验证-Finish')
        
    # =========== return =============          
    return raw_data


import setuptools

with open("./README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="phMaxJobs", # Replace with your own username
    version="0.1.1",
    author="Alfred Yang",
    author_email="alfredyang@pharbers.com",
    description="pharbers dag scheduler config",
    long_description=long_description,
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=2.7',
)



# -*- coding: utf-8 -*-
"""alfredyang@pharbers.com.

This is job template for Pharbers Max Job
"""
import numpy as np
import pandas as pd

from pyspark.sql import SparkSession
import time
from pyspark.sql.types import *
from pyspark.sql.types import StringType, IntegerType, DoubleType
from pyspark.sql import functions as func


def execute(a, b):
	spark = SparkSession.builder \
		.master("yarn") \
		.appName("sparkOutlier") \
		.config("spark.driver.memory", "1g") \
		.config("spark.executor.cores", "1") \
		.config("spark.executor.instance", "2") \
		.config("spark.executor.memory", "2g") \
		.getOrCreate()
		
	# =========== 数据检查 =============
		
	
	# 输入
	product_mapping_out_path = "/user/ywyuan/max/Sankyo/raw_data_job2_out"
	products_of_interest_path = "/workspace/BP_Max_AutoJob/Sankyo/poi.xlsx"
	model_month_right = 201912
	project_name = "Sankyo"
	max_month = 12
	year_missing = []
	
	# 输出
	price_path = "/user/ywyuan/max/Sankyo/price"
	raw_data_adding_path = "/user/ywyuan/max/Sankyo/raw_data_adding"
	new_hospital_path = '/workspace/BP_Max_AutoJob/Sankyo/2019new_hospital.xlsx'
	
	# =========== 数据执行 =============
    logger.info('数据执行-start')

	raw_data = spark.read.parquet(product_mapping_out_path)
	
	products_of_interest = pd.read_excel(products_of_interest_path)
	products_of_interest = products_of_interest["poi"].values.tolist()
	
	# 新增一列S_Molecule_for_gr：products_of_interest为商品名，其他产品为分子名
	raw_data = raw_data.withColumn("S_Molecule_for_gr",
	                               func.when(raw_data["标准商品名"].isin(products_of_interest), raw_data["标准商品名"]).
	                               otherwise(raw_data.S_Molecule))
	
	# 1 价格计算：cal_price 补数部分的数量需要用价格得出
	price = raw_data.groupBy("min2", "year_month", "City_Tier_2010") \
	    .agg((func.sum("Sales") / func.sum("Units")).alias("Price"))
	price2 = raw_data.groupBy("min2", "year_month") \
	    .agg((func.sum("Sales") / func.sum("Units")).alias("Price2"))
	price = price.join(price2, on=["min2", "year_month"], how="left")
	price = price.withColumn("Price", func.when(func.isnull(price.Price), price.Price2).
	                         otherwise(price.Price))
	price = price.withColumn("Price", func.when(func.isnull(price.Price), func.lit(0)).
	                         otherwise(price.Price)) \
	    .drop("Price2")
	
	# 输出price
	price = price.repartition(2)
	price.write.format("parquet") \
	    .mode("overwrite").save(price_path)
	
	raw_data = raw_data.where(raw_data.Year < ((model_month_right // 100) + 1))
	if project_name == "Sanofi" or project_name == "AZ":
	    raw_data = raw_data.where(raw_data.Year > 2016 & raw_data.Year < 2020)

	# 2 计算样本医院连续性: cal_continuity
	# 每个医院每年的月份数
	continuity = raw_data.select("Year", "Month", "PHA").distinct() \
	    .groupBy("PHA", "Year").count()
	# 每个医院最大月份数，最小月份数
	continuity_whole_year = continuity.groupBy("PHA") \
	    .agg(func.max("count").alias("MAX"), func.min("count").alias("MIN"))
	continuity = continuity.repartition(2, "PHA")
	
	years = continuity.select("Year").distinct().toPandas()["Year"].sort_values().values.tolist()
	# 数据长变宽
	continuity = continuity.groupBy("PHA").pivot("Year").agg(func.sum('count')).fillna(0)
	# 列名修改
	for eachyear in years:
	    eachyear = str(eachyear)
	    continuity = continuity.withColumn(eachyear, continuity[eachyear].cast(DoubleType())) \
	        .withColumnRenamed(eachyear, "Year_" + eachyear)
	# year列求和
	# month_sum = con.Year_2018 + con.Year_2019
	month_sum = ""
	for i in continuity.columns[1:]:
	    month_sum += ("continuity." + i + "+")
	month_sum = month_sum.strip('+')
	continuity = continuity.withColumn("total", eval(month_sum))
	# 最大最小值
	# ['PHA', 'Year_2018', 'Year_2019', 'total', 'MAX', 'MIN']
	continuity = continuity.join(continuity_whole_year, on="PHA", how="left")
	
	# 3 计算样本分子增长率: cal_growth
	def calculate_growth(raw_data, max_month=12):
	    # TODO: 完整年用完整年增长，不完整年用不完整年增长
	    if max_month < 12:
	        raw_data = raw_data.where(raw_data.Month <= max_month)
	
	    growth_raw_data = raw_data.na.fill({"City_Tier_2010": 5.0})
	    growth_raw_data = growth_raw_data.withColumn("CITYGROUP", growth_raw_data.City_Tier_2010)
	
	    # 增长率计算过程
	    growth_calculating = growth_raw_data.groupBy("S_Molecule_for_gr", "CITYGROUP", "Year") \
	        .agg(func.sum(growth_raw_data.Sales).alias("value"))
	
	    years = growth_calculating.select("Year").distinct().toPandas()["Year"].sort_values().values.tolist()
	    years = [str(i) for i in years]
	    years_name = ["Year_" + i for i in years]
	    # 数据长变宽
	    growth_calculating = growth_calculating.groupBy("S_Molecule_for_gr", "CITYGROUP").pivot("Year").agg(func.sum('value')).fillna(0)
	    growth_calculating = growth_calculating.select(["S_Molecule_for_gr", "CITYGROUP"] + years)
	    # 对year列名修改
	    for i in range(0, len(years)):
	        growth_calculating = growth_calculating.withColumnRenamed(years[i], years_name[i])
	
	    # 计算得到年增长： add_gr_cols
	    for i in range(0, len(years) - 1):
	        growth_rate = growth_calculating.withColumn("GR" + years[i][2:4] + years[i + 1][2:4],
	                                                    growth_calculating[years_name[i + 1]] / growth_calculating[years_name[i]])
	    # 增长率的调整：modify_gr
	    for y in [name for name in growth_rate.columns if name.startswith("GR")]:
	        growth_rate = growth_rate.withColumn(y, func.when(func.isnull(growth_rate[y]) | (growth_rate[y] > 10) | (growth_rate[y] < 0.1), 1).
	                                             otherwise(growth_rate[y]))
	    return growth_rate
	
	
	# AZ-Sanofi 要特殊处理
	if project_name != "Sanofi" and project_name != "AZ":
	    growth_rate = calculate_growth(raw_data)
	else:
	    # 完整年
	    growth_rate_p1 = calculate_growth(raw_data.where(raw_data.Year.isin(year_missing)))
	    # 不完整年
	    growth_rate_p2 = calculate_growth(raw_data.where(raw_data.Year.isin(year_missing + [y - 1 for y in year_missing] + [y + 1 for y in year_missing])), max_month)
	
	    growth_rate = growth_rate_p1.select("S_Molecule_for_gr", "CITYGROUP") \
	        .union(growth_rate_p2.select("S_Molecule_for_gr", "CITYGROUP")) \
	        .distinct()
	    growth_rate = growth_rate.join(
	        growth_rate_p1.select("S_Molecule_for_gr", "CITYGROUP",[name for name in growth_rate_p1.columns if name.startswith("GR")]),
	        on=["S_Molecule_for_gr", "CITYGROUP"],
	        how="left")
	    growth_rate = growth_rate.join(
	        growth_rate_p2.select("S_Molecule_for_gr", "CITYGROUP",[name for name in growth_rate_p2.columns if name.startswith("GR")]),
	        on=["S_Molecule_for_gr", "CITYGROUP"],
	        how="left")
	
	# 输出growth_rate结果
	growth_rate = growth_rate.repartition(2)
	growth_path_online = "/user/ywyuan/max/Sankyo/gr"
	growth_rate.write.format("parquet") \
	    .mode("overwrite").save(growth_path_online)
	    
	# 4 补数: 
	# 4.1 原始数据格式整理: trans_raw_data_for_adding
	growth_rate = growth_rate.select(["CITYGROUP", "S_Molecule_for_gr"] + [name for name in growth_rate.columns if name.startswith("GR1")]) \
	    .distinct()
	raw_data_for_add = raw_data.where(raw_data.PHA.isNotNull()) \
	    .orderBy(raw_data.Year.desc()) \
	    .withColumnRenamed("City_Tier_2010", "CITYGROUP") \
	    .join(growth_rate, on=["S_Molecule_for_gr", "CITYGROUP"], how="left")

	# 1.7 补充各个医院缺失的月份:
	# add_data
	original_range = seed.select("Year", "Month", "PHA").distinct()
	
	years = original_range.select("Year").distinct() \
	    .orderBy(original_range.Year) \
	    .toPandas()["Year"].values.tolist()
	print(years)
	
	all_gr_index = [index for index, name in enumerate(seed.columns) if name.startswith("GR")]
	print(all_gr_index)
	
	# 每年的补数
	# price_path = "/common/projects/max/Sankyo/price"
	#price_path = "/user/ywyuan/max/Sankyo/price"
	#price = spark.read.parquet(price_path)
	
	# 往下测试
	#years = [2018, 2019]
	#all_gr_index = [31]
	#original_range = spark.read.parquet("/user/ywyuan/max/Sankyo/original_range")
	#seed = spark.read.parquet("/user/ywyuan/max/Sankyo/seed")
	
	empty = 0
	for eachyear in years:
	    # cal_time_range
	    # 当前年的 月份-PHA 集合
	    current_range_pha_month = original_range.where(original_range.Year == eachyear) \
	        .select("Month", "PHA").distinct()
	    # 当前年的 月份 集合
	    current_range_month = current_range_pha_month.select("Month").distinct()
	    # 其他年 在当前年有的 月份-PHA
	    other_years_range = original_range.where(original_range.Year != eachyear) \
	        .join(current_range_month, on="Month", how="inner") \
	        .join(current_range_pha_month, on=["Month", "PHA"], how="left_anti")
	    # 其他年 与 当前年的 差值，比重计算
	    other_years_range = other_years_range \
	        .withColumn("time_diff", (other_years_range.Year - eachyear)) \
	        .withColumn("weight", func.when((other_years_range.Year > eachyear), (other_years_range.Year - eachyear - 0.5)).
	                    otherwise(other_years_range.Year * (-1) + eachyear))
	    # 选择比重最小的其他年份
	    seed_range = other_years_range.orderBy(other_years_range.weight) \
	        .groupBy("PHA", "Month") \
	        .agg(func.first(other_years_range.Year).alias("Year"))
	
	    # get_seed_data
	    # 从rawdata根据seed_range获取要补数的数据
	    seed_for_adding = seed.where(seed.Year != eachyear) \
	        .join(seed_range, on=["Month", "PHA", "Year"], how="inner")
	    seed_for_adding = seed_for_adding \
	        .withColumn("time_diff", (seed_for_adding.Year - eachyear)) \
	        .withColumn("weight", func.when((seed_for_adding.Year > eachyear), (seed_for_adding.Year - eachyear - 0.5)).
	                    otherwise(seed_for_adding.Year * (-1) + eachyear))
	
	    # cal_seed_with_gr
	    base_index = eachyear - min(years) + min(all_gr_index)
	    seed_for_adding = seed_for_adding.withColumn("Sales_bk", seed_for_adding.Sales)
	
	    # min_index：seed_for_adding年份小于当前年， time_diff+base_index
	    # max_index：seed_for_adding年份小于当前年，base_index-1
	    seed_for_adding = seed_for_adding \
	        .withColumn("min_index", func.when((seed_for_adding.Year < eachyear), (seed_for_adding.time_diff + base_index)).
	                    otherwise(base_index)) \
	        .withColumn("max_index", func.when((seed_for_adding.Year < eachyear), (base_index - 1)).
	                    otherwise(seed_for_adding.time_diff + base_index - 1)) \
	        .withColumn("total_gr", func.lit(1))
	
	    for i in all_gr_index:
	        col_name = seed_for_adding.columns[i]
	        seed_for_adding = seed_for_adding.withColumn(col_name, func.when(
	            (seed_for_adding.min_index > i) | (seed_for_adding.max_index < i), 1).
	                                                     otherwise(seed_for_adding[col_name]))
	        seed_for_adding = seed_for_adding.withColumn(col_name, func.when(seed_for_adding.Year > eachyear,
	                                                                         seed_for_adding[col_name] ** (-1)).
	                                                     otherwise(seed_for_adding[col_name]))
	        seed_for_adding = seed_for_adding.withColumn("total_gr", seed_for_adding.total_gr * seed_for_adding[col_name])
	
	    seed_for_adding = seed_for_adding.withColumn("final_gr",
	                                                 func.when(seed_for_adding.total_gr < 2, seed_for_adding.total_gr).
	                                                 otherwise(2))
	    seed_for_adding = seed_for_adding \
	        .withColumn("Sales", seed_for_adding.Sales * seed_for_adding.final_gr) \
	        .withColumn("Year", func.lit(eachyear))
	    seed_for_adding = seed_for_adding.withColumn("year_month", seed_for_adding.Year * 100 + seed_for_adding.Month)
	    seed_for_adding = seed_for_adding.withColumn("year_month", seed_for_adding["year_month"].cast(DoubleType()))
	
	    seed_for_adding = seed_for_adding.withColumnRenamed("CITYGROUP", "City_Tier_2010") \
	        .join(price, on=["min2", "year_month", "City_Tier_2010"], how="inner")
	    seed_for_adding = seed_for_adding.withColumn("Units", func.when(seed_for_adding.Sales == 0, 0).
	                                                 otherwise(seed_for_adding.Sales / seed_for_adding.Price)) \
	        .na.fill({'Units': 0})
	
	    if empty == 0:
	        adding_data = seed_for_adding
	    else:
	        adding_data = adding_data.union(seed_for_adding)
	    empty = empty + 1
	

	# 1.8 合并补数部分和原始部分:
	# combind_data
	raw_data_adding = (raw_data.withColumn("add_flag", func.lit(0))) \
	    .union(adding_data.withColumn("add_flag", func.lit(1)).select(raw_data.columns + ["add_flag"]))
	# 输出第一阶段补数结果
	raw_data_adding = raw_data_adding.repartition(2)
	raw_data_adding.write.format("parquet") \
		.mode("overwrite").save(raw_data_adding_path)
	
	# 1.9 进一步为最后一年独有的医院补最后一年的缺失月（可能也要考虑第一年）:
	
	#      （可能也要考虑第一年）:
	print(years)
	
	# 只在最新年份出现的PHA-医院
	new_hospital = (original_range.where(original_range.Year == max(years)).select("PHA").distinct()) \
	    .subtract(original_range.where(original_range.Year != max(years)).select("PHA").distinct()) \
	    .toPandas()
	print("以下是最新一年出现的医院:" + str(new_hospital["PHA"].tolist()))
	new_hospital.to_excel(new_hospital_path)
	
	# 最新年份没有的月份
	missing_months = (original_range.where(original_range.Year != max(years)).select("Month").distinct()) \
	    .subtract(original_range.where(original_range.Year == max(years)).select("Month").distinct())
	
	# 如果最新年份有缺失月份，特殊处理
	if missing_months.count() == 0:
	    adding_data_new = raw_data_adding
	else:
	    number_of_existing_months = 12 - missing_months.count()
	    group_cols = set(raw_data_adding.columns).difference(
	        set(['Month', 'Sales', 'Units', '季度', "sales_value__rmb_", "total_units", "counting_units", "year_month"]))
	    adding_data_new = raw_data_adding \
	        .where(raw_data_adding.add_flag == 1) \
	        .where(raw_data_adding.PHA.isin(new_hospital["PHA"].tolist())) \
	        .groupBy(list(group_cols)).agg({"Sales": "sum", "Units": "sum"})
	    adding_data_new = adding_data_new \
	        .withColumn("Sales", adding_data_new["sum(Sales)"] / number_of_existing_months) \
	        .withColumn("Units", adding_data_new["sum(Units)"] / number_of_existing_months) \
	        .crossJoin(missing_months)
	    same_names = list(set(raw_data_adding.columns).intersection(set(adding_data_new.columns)))
	    adding_data_new = raw_data_adding.select(same_names) \
	        .union(adding_data_new.select(same_names))
	
	# 输出补数结果 adding_data_new
	adding_data_new.repartition(2).write.format("parquet") \
	    .mode("overwrite").save("/user/ywyuan/max/Sankyo/adding_data_new")
	adding_data_new.show(2)

		



from phjob import execute
import click


@click.command()
@click.option('--max_path')
@click.option('--max_path_local')
@click.option('--project_name')
@click.option('--minimum_product_columns')
@click.option('--minimum_product_sep')
@click.option('--minimum_product_newname')
@click.option('--need_cleaning_cols')
@click.option('--test_out_path')
def debug_execute(max_path, max_path_local, project_name, minimum_product_columns, minimum_product_sep, minimum_product_newname, need_cleaning_cols, test_out_path):
    execute(max_path, max_path_local, project_name, minimum_product_columns, minimum_product_sep, minimum_product_newname, need_cleaning_cols, test_out_path)


if __name__ == '__main__':
    debug_execute()
